#! /usr/bin/env python

import gd, sys, time
from math import exp, sin, cos, pi, log
import os
import sys

class mandelbrotSet:

    def __init__(self):
        print 'created Mandelbrot set'

    def iter(self, z):
        c = z
        for i in range(maxIter):
            z = z*z + c
            if abs(z) > bailout:
                return i
        return maxIter

class graph:

    center = complex(0,0)
    width  = 2.0
    pixels = 100
    canvas = gd.image((pixels, pixels))
    fractalSet = mandelbrotSet()

    def gauss(self, x, mu, sigma):
        return int(255*exp(-(x-mu)*(x-mu)/(sigma*sigma)))

    def setColors(self):
        for i in range(nColors):
            r = self.gauss(i,  75.0, 60.0)
            g = self.gauss(i, 125.0, 60.0)
            b = self.gauss(i, 180.0, 60.0)
            self.canvas.colorAllocate((r, g, b))

    def setWidth(self, twidth):
        self.width = twidth

    def plot(self):
        ti = time.clock()
        dx = self.width/self.pixels
        bl = self.center - complex(self.width/2, self.width/2)
        print 'plotting graph: center:', self.center, 'width:', self.width
        print 'starting row',
        for ix in range(self.pixels):
            if (ix % (self.pixels/10) == 0):
                print ix,
                sys.stdout.flush()
            for iy in range(self.pixels):
                c = bl + complex(ix*dx, iy*dx)
                color = self.fractalSet.iter(c)
                self.canvas.setPixel((ix, self.pixels-1 - iy), color)
        print '...done'
        self.canvas.writeJpeg(jpgFile, 95)
        tf = time.clock()
        dt = round(tf-ti, 2)
        print 'file', jpgFile, 'has been created in', dt, 'seconds.'

    def __init__(self, tset, tpixels=100, twidth=3.0, tcenter=complex(0,0)):
        self.center = tcenter
        self.width = twidth
        self.pixels = tpixels
        self.fractalSet = tset
        self.canvas = gd.image((self.pixels, self.pixels))
        self.setColors()
        print 'created graph: size', self.pixels, 'pixels' 

# main ...

print "fractals by nicZ"

bailout = 2.0
maxIter = 250
nColors = maxIter + 1

center = complex(-1.27966, 0.064341)
wmax = 3.7
wmin = 0.0015
nFrames = 240
k = exp((log(wmin)-log(wmax))/float(nFrames))
print 'zoom factor is', k

mySet = mandelbrotSet()
myGraph = graph(mySet, 500, wmax, center)

for i in range(nFrames):
    jpgFile = 'frame%03i.jpg' % i
    if os.path.exists(jpgFile):
        print jpgFile, 'already exists.'
    else:
        w = wmax*(k**(i))
        myGraph.setWidth(w)
        myGraph.plot()
        print
        
    
    



